#!/bin/bash

# Build the base images used in the Gitlab CI/CD pipeline.

# Two docker images are created:
#  - earthworm-<BASE_LINUX_SLUG>-base:build
#  - earthworm-<BASE_LINUX_SLUG>-base:ops
#
# The <BASE_LINUX_SLUG> has to be specified on the command line.
#
# The two images have to be pushed to the Gitlab container registry.
# To push the images to the registry, a project access token with
# the rights to read and write the container registry has to be created.
# This can be done in the Gitlab interface under "Settings->Access Tokens".
# Add a token with the following options activated:
#  - read_registry
#  - write_registry
#
# Make sure, that the role is set to one with sufficient rights to write
# to the registry.
# The token ID is used as the password for the login. The access token is only
# visible once after creation, so write it down.
#
# To push the image to the container registry first login using the access token.
# docker login registry.gitlab.com -u USERNAME
# Use the access token ID as the password.
#
# Next push the two created images.
# docker push registry.gitlab.com/seismic-software/earthworm/earthworm-rockylinux8_9-base:build 
# docker push registry.gitlab.com/seismic-software/earthworm/earthworm-rockylinux8_9-base:ops

if [ "$#" -ne 1 ]; then
    printf "Please specify the BASE_LINUX_SLUG.\nUsage: $0 BASE_LINUX_SLUG\n"
    exit 2
fi

BASE_LINUX_SLUG=$1

BUILD_FILE=Dockerfile.${BASE_LINUX_SLUG}.build
OPS_FILE=Dockerfile.${BASE_LINUX_SLUG}.ops

echo "Building the build base image using $BUILD_FILE."
docker build -t registry.gitlab.com/seismic-software/earthworm/earthworm-${BASE_LINUX_SLUG}-base:build -f $BUILD_FILE .
echo "Building the build base image using $OPS_FILE."
docker build -t registry.gitlab.com/seismic-software/earthworm/earthworm-${BASE_LINUX_SLUG}-base:ops -f $OPS_FILE .

