/* 
 * config.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <kom.h>
#include <transport.h>
#include <earthworm.h>
#include "exportfilter.h"
#include "pick_gm.h"
#include <math.h>

#define ncommand 8          /* Number of commands in the config file */

//extern void setResponseInMeters(int flag);

 /***********************************************************************
  *                              GetConfig()                            *
  *             Processes command file using kom.c functions.           *
  *               Returns -1 if any errors are encountered.             *
  ***********************************************************************/

int GetConfig( char *config_file, GPARM *Gparm )
{
	char     init[ncommand];     /* Flags, one for each command */
	int      nmiss;              /* Number of commands that were missed */
	int      nfiles;
	int      i, k, imax;
	double   PSAdt, PSAdt_base;

/* Set to zero one init flag for each required command
	***************************************************/
	for ( i = 0; i < ncommand; i++ ) init[i] = 0;
	Gparm->nGetLogo = 0;
	Gparm->GetLogo  = NULL;
	//Gparm->PickIndexDir  = NULL; /* optional directory for pick index placement */
	//Gparm->NoCoda = 0;		/* off by default, always calculate coda's */
	Gparm->nStaFile = 0;
	Gparm->StaFile  = NULL;
	Gparm->MMISource = 'M';

	Gparm->sniffGmStyle = 0;

	Gparm->dummyLeadSec = 120;

	Gparm->StableTime = 45.0;
	Gparm->SamplePeriod = 60;
	Gparm->WriteGMX = 0; /* off */
	//Gparm->WriteSimFile = 0; /* off */
	Gparm->maxTrace = 0;
	//Gparm->numSCNLPar = 0;
	//Gparm->maxSCNLPar = 0;
	//Gparm->pSCNLPar = NULL;
	//Gparm->TimeTaper = 0.0;
	Gparm->PsaPeriodType = 0;
	Gparm->numPeriods = 0;
	Gparm->PsaMethod = 0; /* FFT */
	Gparm->PsaMultiple = 10.0;
	Gparm->ClipLimit = 7549747; /* 90% of 2^23; clip for 24-bit digitizer */
	Gparm->pgThresh = 0.0;
	//Gparm->peakSearchStart = 0;
	//Gparm->peakSearchEnd = 30;
	//Gparm->lowpoles = Gparm->highpoles = Gparm->lowcutoff = Gparm->highcutoff = 0;
	//Gparm->HorizontalComponentVector = 0;

	//memset(Gparm->PsaFolder, 0, sizeof(Gparm->PsaFolder));
	//memset(Gparm->FasFolder, 0, sizeof(Gparm->FasFolder));

/* Open the main configuration file
	********************************/
	nfiles = k_open( config_file );
	if ( nfiles == 0 )
	{
		logit( "e", "pick_gm: Error opening configuration file <%s>\n",
					config_file );
		return -1;
	}

/* Process all nested configuration files
	**************************************/
	while ( nfiles > 0 )          /* While there are config files open */
	{
		while ( k_rd() )           /* Read next line from active file  */
		{
			int  success;
			char *com;
			char *str;

			com = k_str();          /* Get the first token from line */

			if ( !com ) continue;             /* Ignore blank lines */
			if ( com[0] == '#' ) continue;    /* Ignore comments */

/* Open another configuration file
	*******************************/
			if ( com[0] == '@' )
			{
				success = nfiles + 1;
				nfiles  = k_open( &com[1] );
				if ( nfiles != success )
				{
					logit( "e", "pick_gm: Error opening command file <%s>.\n",
								&com[1] );
					return -1;
				}
				continue;
			}

/* Read configuration parameters
	*****************************/
			else if ( k_its( "StaFile" ) )
			{
				str = k_str();
				if ( str!=NULL  &&  strlen(str)<STAFILE_LEN ) 
				{
					STAFILE *tmp;
					tmp = (STAFILE *)realloc( Gparm->StaFile, (Gparm->nStaFile+1)*sizeof(STAFILE) );
					if( tmp == NULL ) 
					{
						logit( "e", "pick_gm: Error reallocing Gparm->StaFile. Exiting.\n" );
						return -1;
					}
					Gparm->StaFile = tmp;
					strcpy( Gparm->StaFile[Gparm->nStaFile].name, str );
					Gparm->StaFile[Gparm->nStaFile].nsta = 0;
					Gparm->nStaFile++;
				} else {
					logit( "e", "pick_gm: Invalid StaFile name <%s>; must be 1-%d chars. " 
							 "Exiting.\n", str, STAFILE_LEN );
					return -1;
				}
				init[0] = 1;
			}

			else if ( k_its( "InRing" ) )
			{
				if ( (str = k_str()) != NULL )
				{
					if( (Gparm->InKey = GetKey(str)) == -1 )
					{
						logit( "e", "pick_gm: Invalid InRing name <%s>. Exiting.\n", str );
						return -1;
					}
				}
				init[1] = 1;
			}

			else if ( k_its( "OutRing" ) )
			{
				if ( (str = k_str()) != NULL )
				{
					if ( (Gparm->OutKey = GetKey(str)) == -1 )
					{
						logit( "e", "pick_gm: Invalid OutRing name <%s>. Exiting.\n", str );
						return -1;
					}
				}
				init[2] = 1;
			}

			else if ( k_its( "HeartbeatInt" ) )
			{
				Gparm->HeartbeatInt = k_int();
				init[3] = 1;
			}

			else if ( k_its( "RestartLength" ) )
			{
				Gparm->RestartLength = k_int();
				init[4] = 1;
			}

			else if ( k_its( "MaxGap" ) )
			{
				Gparm->MaxGap = k_int();
				init[5] = 1;
			}

			else if ( k_its( "Debug" ) )
			{
				Gparm->Debug = k_int();
				init[6] = 1;
			}

			else if ( k_its( "OutputByFile" ) )
			{
				Gparm->OutputByFile = k_int();
				init[6] = 1;
			}

			else if ( k_its( "OutputByConsole" ) )
			{
				Gparm->OutputByConsole = k_int();
				init[6] = 1;
			}

			else if ( k_its( "MMISource" ) )
			{
				str = k_str();
				if (str !=NULL) Gparm->MMISource = str[0];
			}

			else if ( k_its( "OutputFolder" ) )
			{
				str = k_str();
				if (str != NULL) strcpy( Gparm->OutputFolder, str );
				init[6] = 1;
			}

			else if ( k_its( "MyModId" ) )
			{
				if ( (str = k_str()) != NULL )
				{
					if ( GetModId(str, &Gparm->MyModId) == -1 )
					{
						logit( "e", "pick_gm: Invalid MyModId <%s>.\n", str );
						return -1;
					}
				}
				init[7] = 1;
			}

			else if (k_its("maxTrace"))
			{
				Gparm->maxTrace = (long)k_int();
			}
			else if (k_its("StableTime"))
			{
				Gparm->StableTime = (long)k_val();
			}
			//else if (k_its("SimFolder"))
			//{
			//	str = k_str();
			//	if (str != NULL) strcpy(Gparm->SimFolder, str);
			//	Gparm->WriteSimFile = 1;
			//}
			else if (k_its("SamplePeriod"))
			{
				Gparm->SamplePeriod = k_int();
			}
			/*else if (k_its("WriteGMX"))
			{
				Gparm->WriteGMX = k_int();
			}*/
			else if (k_its("PsaMethod"))
			{
				Gparm->PsaMethod = (k_int() - 1);
				if (Gparm->PsaMethod < 0) Gparm->PsaMethod = 2; /* nigam-jennings */
			}
			/* the responses are in meters, from rdseed -pf, convert them to nanometers when read in */
			//else if (k_its("ResponseInMeters"))
			//{
			//	setResponseInMeters(1);
			//}
			//else if (k_its("RespDir"))
			//{
			//	str = k_str();
			//	if (str != NULL) strcpy(Gparm->RespDir, str);
			//}
			//else if (k_its("RespNameFormat"))
			//{
			//	str = k_str();
			//	if (str != NULL) strcpy(Gparm->RespNameFormat, str);
			//}
			//else if (k_its("TimeTaper"))
			//{
			//	Gparm->TimeTaper = k_val();
			//}
			/* SCNL Parameters: optional */
			/*else if (k_its("SCNLpar"))
			{
				if (Gparm->numSCNLPar >= Gparm->maxSCNLPar)
				{
					Gparm->maxSCNLPar += 10;
					if ((Gparm->pSCNLPar =
						(SCNLPAR *)realloc(Gparm->pSCNLPar, sizeof(SCNLPAR) *
							Gparm->maxSCNLPar)) == (SCNLPAR *)NULL)
					{
						fprintf(stderr, "ReadConfig: out of memory for %d SCNLPARs\n",
							Gparm->maxSCNLPar);
						return -1;
					}
				}
				if ((str = k_str()) == NULL)
				{
					fprintf(stderr, "ReadConfig: Bad \"SCNLPar\" command\n");
					return -1;
				}
				strncpy(Gparm->pSCNLPar[Gparm->numSCNLPar].sta, str, TRACE_STA_LEN);
				if ((str = k_str()) == NULL)
				{
					fprintf(stderr, "ReadConfig: Bad \"SCNLPar\" command\n");
					return -1;
				}
				strncpy(Gparm->pSCNLPar[Gparm->numSCNLPar].chan, str, TRACE_CHAN_LEN);
				if ((str = k_str()) == NULL)
				{
					fprintf(stderr, "ReadConfig: Bad \"SCNLPar\" command\n");
					return -1;
				}
				strncpy(Gparm->pSCNLPar[Gparm->numSCNLPar].net, str, TRACE_NET_LEN);
				if ((str = k_str()) == NULL)
				{
					fprintf(stderr, "ReadConfig: Bad \"SCNLPar\" command\n");
					return -1;
				}
				strncpy(Gparm->pSCNLPar[Gparm->numSCNLPar].loc, str, TRACE_LOC_LEN);
				Gparm->pSCNLPar[Gparm->numSCNLPar].fTaper[0] = k_val();
				Gparm->pSCNLPar[Gparm->numSCNLPar].fTaper[1] = k_val();
				Gparm->pSCNLPar[Gparm->numSCNLPar].fTaper[2] = k_val();
				Gparm->pSCNLPar[Gparm->numSCNLPar].fTaper[3] = k_val();
				Gparm->pSCNLPar[Gparm->numSCNLPar].clipLimit = k_val();
				Gparm->pSCNLPar[Gparm->numSCNLPar].taperTime = k_val();
				Gparm->numSCNLPar++;
			}*/
			/* PSA Periods: point or range(optional) */
			else if (k_its("PsaPeriods"))
			{
				Gparm->numPeriods = 0;
				if ((str = k_str()))
				{
					if (k_its("point"))
					{

						Gparm->PsaPeriodType = 0;
						while (Gparm->numPeriods < MAX_PERIODS && (Gparm->PsaPeriods[Gparm->numPeriods] = k_val())) Gparm->numPeriods++;

					}
					else if (k_its("range"))
					{
						Gparm->PsaPeriodType = 1;
						Gparm->PsaPeriods[0] = k_val();
						Gparm->PsaPeriods[1] = k_val();
						PSAdt = k_val();

						Gparm->numPeriods = (int)((Gparm->PsaPeriods[1] - Gparm->PsaPeriods[0]) / PSAdt + 1.5);
						if (Gparm->numPeriods > MAX_PERIODS) Gparm->numPeriods = MAX_PERIODS;
						for (i = 1; i < Gparm->numPeriods; i++)
							Gparm->PsaPeriods[i] = Gparm->PsaPeriods[0] + (i * PSAdt);
					}
					else if (k_its("log"))
					{
						int s_scale, e_scale, d_scale;
						double psaper;

						Gparm->PsaPeriodType = 2;
						s_scale = k_int();
						e_scale = k_int();
						d_scale = k_int();

						PSAdt_base = pow(10.0, s_scale);;
						imax = e_scale - s_scale;
						for (i = 0; i <= imax; i++) {
							PSAdt = pow(10.0, (s_scale + 1)) * pow(10.0, i);
							for (k = 0; k < d_scale - 1; k++) {
								psaper = PSAdt_base + (k * PSAdt / d_scale);
								if (log10(psaper) >= log10(PSAdt)) continue;

								Gparm->PsaPeriods[Gparm->numPeriods++] = psaper;
								if (i == imax) break;
							}
							PSAdt_base = PSAdt;
						}
					}
					else
					{
						logit("e", "ReadConfig: unknown \"PsaPeriod\" <%s>\n", str);
						return -1;
					}
				}
				else
				{
					logit("e", "ReadConfig: \"PsaPeriod\" missing argument\n");
					return -1;
				}
			}
			//else if (k_its("PsaFolder"))
			//{
			//	str = k_str();
			//	if (str != NULL) strcpy(Gparm->PsaFolder, str);
			//}
			else if (k_its("ClipLimit"))
			{
				Gparm->ClipLimit = k_int();
			}
			//else if (k_its("SearchWindow"))
			//{
			//	Gparm->peakSearchStart = k_val();
			//	Gparm->peakSearchEnd = k_val();
			//}
			//else if (t_com());

			//else if (k_its("FasFolder"))
			//{
			//	str = k_str();
			//	if (str != NULL) strcpy(Gparm->FasFolder, str);
			//}
			//else if (k_its("HighCut")) {
			//	Gparm->highcutoff = k_val();
			//	if (Gparm->highcutoff <= 0) {
			//		logit("e", "HighCut cutoff frequency must be > 0\n");
			//		return -1;
			//	}
			//	Gparm->highpoles = k_int();
			//	if (Gparm->highpoles % 2 || Gparm->highpoles < 2 || Gparm->highpoles > 12) {
			//		logit("e", "# poles for HighCut must be 2, 4, 6, 8, 10 or 12\n");
			//		return -1;
			//	}
			//}
			//else if (k_its("LowCut")) {
			//	Gparm->lowcutoff = k_val();
			//	if (Gparm->lowcutoff <= 0) {
			//		logit("e", "LowCut cutoff frequency must be > 0\n");
			//		return -1;
			//	}
			//	Gparm->lowpoles = k_int();
			//	if (Gparm->lowpoles % 2 || Gparm->lowpoles < 2 || Gparm->lowpoles > 12) {
			//		logit("e", "# poles for LowCut must be 2, 4, 6, 8, 10 or 12\n");
			//		return -1;
			//	}
			//}
			//else if (k_its("HorizontalComponentVector"))
			//{
			//	Gparm->HorizontalComponentVector = k_int();
			//}

			/*opt*/ else if ( k_its( "NoCoda" ) )
			{
				//Gparm->NoCoda = k_int();
			}
			/*opt*/ else if ( k_its( "PickIndexDir" ) )
			{
				//Gparm->PickIndexDir = strdup(k_str());
			}
			/*opt*/ else if ( k_its( "WeightTable" ) )
			{
				//Gparm->WeightTable[0] = (float) k_val();
				//Gparm->WeightTable[1] = (float) k_val();
				//Gparm->WeightTable[2] = (float) k_val();
				//Gparm->WeightTable[3] = (float) k_val();
			}
			/*opt*/ else if ( k_its( "SniffGmStyle" ) )
			{
				Gparm->sniffGmStyle = k_int();
			}
			/*opt*/ else if (k_its("DummyLeadSec"))
			{
				Gparm->dummyLeadSec = k_int();
			}
			/*opt*/ else if ( k_its( "pgThresh" ) )
                        {
			    Gparm->pgThresh = k_val();
			}
			/*opt*/ else if ( k_its( "GetLogo" ) )
			{
				MSG_LOGO *tlogo = NULL;
				int       nlogo = Gparm->nGetLogo;
				tlogo = (MSG_LOGO *)realloc( Gparm->GetLogo, (nlogo+1)*sizeof(MSG_LOGO) );
				if( tlogo == NULL )
				{
					logit( "e", "pick_gm: GetLogo: error reallocing %d bytes.\n",
							 (nlogo+1)*sizeof(MSG_LOGO) );
					return -1;
				}
				Gparm->GetLogo = tlogo;
				
				if( (str=k_str()) != NULL )       /* read instid */
				{
					if( GetInst( str, &(Gparm->GetLogo[nlogo].instid) ) != 0 ) 
					{
						logit( "e", "pick_gm: Invalid installation name <%s>"
								 " in <GetLogo> cmd!\n", str );
						return -1;
					}
					if( (str=k_str()) != NULL )    /* read module id */
					{
						if( GetModId( str, &(Gparm->GetLogo[nlogo].mod) ) != 0 ) 
						{
							logit( "e", "pick_gm: Invalid module name <%s>"
									 " in <GetLogo> cmd!\n", str );
							return -1;
						}
						if( (str=k_str()) != NULL ) /* read message type */
						{
							if( strcmp(str,"TYPE_TRACEBUF") !=0 &&
								 strcmp(str,"TYPE_TRACEBUF2")!=0    )
							{
								logit( "e","pick_gm: Invalid message type <%s> in <GetLogo>"
										 " cmd; must be TYPE_TRACEBUF or TYPE_TRACEBUF2.\n",
										  str );
								return -1;
							}
							if( GetType( str, &(Gparm->GetLogo[nlogo].type) ) != 0 ) {
								logit( "e", "pick_gm: Invalid message type <%s>"
										 " in <GetLogo> cmd!\n", str );
								return -1;
							}
							Gparm->nGetLogo++;
						} /* end if msgtype */
					} /* end if modid */  
				} /* end if instid */  
			} /* end GetLogo cmd */
  			else if( exportfilter_com() ) {
				//strcpy( processor, "exportfilter_com" );
			}
/* An unknown parameter was encountered
	************************************/
			else
			{
				logit( "e", "pick_gm: <%s> unknown parameter in <%s>\n",
						  com, config_file );
				continue;
			}

/* See if there were any errors processing the command
	***************************************************/
			if (Gparm->PsaPeriodType != 0) {
				if (k_err())
				{
					logit("e", "pick_gm: Bad <%s> command in <%s>.\n", com,
						config_file);
					return -1;
				}
			}
		}
		nfiles = k_close();
	}

/* After all files are closed, check flags for missed commands
	***********************************************************/
	nmiss = 0;
	for ( i = 0; i < ncommand; i++ )
		if ( !init[i] )
			nmiss++;

	if ( nmiss > 0 )
	{
		logit( "e", "pick_gm: ERROR, no " );
		if ( !init[0] ) logit( "e", "<StaFile> " );
		if ( !init[1] ) logit( "e", "<InRing> " );
		if ( !init[2] ) logit( "e", "<OutRing> " );
		if ( !init[3] ) logit( "e", "<HeartbeatInt> " );
		if ( !init[4] ) logit( "e", "<RestartLength> " );
		if ( !init[5] ) logit( "e", "<MaxGap> " );
		if ( !init[6] ) logit( "e", "<Debug> " );
		if ( !init[7] ) logit( "e", "<MyModId> " );
		logit( "e", "command(s) in <%s>. Exiting.\n", config_file );
		return -1;
	}

	// init variable
	Gparm->fpGM = NULL;
	Gparm->fpGMX = NULL;

	return 0;
}


 /***********************************************************************
  *                              LogConfig()                            *
  *                                                                     *
  *                   Log the configuration parameters                  *
  ***********************************************************************/

void LogConfig( GPARM *Gparm )
{
	int i;
	//char wtable[100];
	//SCNLPAR *pSCNLPar;

	logit( "", "\n" );
	logit( "", "nStaFile:        %6d\n",   Gparm->nStaFile );
	for( i=0; i<Gparm->nStaFile; i++ ) {
		logit( "", "StaFile[%d]:    %s\n",  i, Gparm->StaFile[i].name );
	}
	logit( "", "InKey:           %6d\n",   Gparm->InKey );
	logit( "", "OutKey:          %6d\n",   Gparm->OutKey );
	logit( "", "HeartbeatInt:    %6d\n",   Gparm->HeartbeatInt );
	logit( "", "RestartLength:   %6d\n",   Gparm->RestartLength );
	logit( "", "MaxGap:          %6d\n",   Gparm->MaxGap );

	//wtable[0] = '\0';
	//for( i=0; i<=3; i++) {
	//   sprintf(wtable, "%s %.3f", wtable, Gparm->WeightTable[i]);
	//}
	//logit( "", "WeightTable:     %s\n", wtable); 

	logit( "", "Debug:           %6d\n",   Gparm->Debug );
	logit( "", "MMISource:       %c\n",   Gparm->MMISource );
	logit( "", "SniffGmStype:    %6d\n",   Gparm->sniffGmStyle);
	logit( "", "MyModId:         %6u\n",   Gparm->MyModId );
	logit( "", "nGetLogo:        %6d\n",   Gparm->nGetLogo );
	for( i=0; i<Gparm->nGetLogo; i++ ) {
		logit( "", "GetLogo[%d]:   i%u m%u t%u\n", i,
				Gparm->GetLogo[i].instid, Gparm->GetLogo[i].mod, 
				Gparm->GetLogo[i].type );
	}

	logit("", "StableTime   : %.1f\n", Gparm->StableTime);
	logit("", "SamplePeriod : %d\n", Gparm->SamplePeriod);
	logit("", "PsaPeriodType: %d\n", Gparm->PsaPeriodType);
	logit("", "numPeriods   : %d\n", Gparm->numPeriods);
	//logit("", "numSCNLPar   : %d\n", Gparm->numSCNLPar);

	/*for (i = 0; i < Gparm->numSCNLPar; i++) {
		pSCNLPar = Gparm->pSCNLPar + i;
		logit("", "%s %s %s %s %.2f %.2f %.2f %.2f %g %.2f\n",
			pSCNLPar->sta, pSCNLPar->chan, pSCNLPar->net, pSCNLPar->loc,
			pSCNLPar->fTaper[0], pSCNLPar->fTaper[1], pSCNLPar->fTaper[2], pSCNLPar->fTaper[3],
			pSCNLPar->clipLimit, pSCNLPar->taperTime);
	}*/

	return;
}

